import os
import subprocess
import urllib2
import shutil
import logging
import re

from django.core.management.commands import startproject
from zc.buildout import UserError
import zc.recipe.egg
import setuptools

script_template = {
    'wsgi': '''

%(relative_paths_setup)s
import sys
sys.path[0:0] = [
  %(path)s,
  ]
%(initialization)s
import %(module_name)s

application = %(module_name)s.%(attrs)s(%(arguments)s)
''',
    'fcgi': '''

%(relative_paths_setup)s
import sys
sys.path[0:0] = [
  %(path)s,
  ]
%(initialization)s
import %(module_name)s

%(module_name)s.%(attrs)s(%(arguments)s)
'''
}

production_settings = '''
from %(project)s.settings import *
'''

development_settings = '''
from %(project)s.settings import *

DEBUG = True
TEMPLATE_DEBUG = DEBUG
'''

urls_template = '''
from django.conf.urls.defaults import patterns, include, handler500
from django.conf import settings
from django.contrib import admin
admin.autodiscover()

handler500 # Pyflakes

urlpatterns = patterns(
    '',
    (r'^admin/(.*)', admin.site.root),
    (r'^accounts/login/$', 'django.contrib.auth.views.login'),
)

if settings.DEBUG:
    urlpatterns += patterns('',
        (r'^media/(?P<path>.*)$', 'django.views.static.serve',
         {'document_root': settings.MEDIA_ROOT}),
    )
'''

class Recipe(object):
    def __init__(self, buildout, name, options):
        self.log = logging.getLogger(name)
        eggs = options.get('eggs', '')
        has_django = re.findall(r'(?<![^\s])django[\s=$]', eggs, re.MULTILINE | re.UNICODE | re.IGNORECASE)
        if not has_django:
            eggs = 'Django\n%s' % eggs
            options['eggs'] = eggs
        self.egg = zc.recipe.egg.Egg(buildout, options['recipe'], options)

        self.buildout, self.name, self.options = buildout, name, options
        options['bin-directory'] = buildout['buildout']['bin-directory']

        options.setdefault('project', 'project')
        options.setdefault('settings', 'development')

        options.setdefault('urlconf', options['project'] + '.urls')
        options.setdefault(
            'media_root',
            "os.path.join(os.path.dirname(__file__), 'media')")
        # Set this so the rest of the recipe can expect the values to be
        # there. We need to make sure that both pythonpath and extra-paths are
        # set for BBB.
        if 'extra-paths' in options:
            options['pythonpath'] = options['extra-paths']
        else:
            options.setdefault('extra-paths', options.get('pythonpath', ''))

        # Usefull when using archived versions
        buildout['buildout'].setdefault(
            'download-cache',
            os.path.join(buildout['buildout']['directory'],
                         'downloads'))

        # mod_wsgi support script
        options.setdefault('wsgi', 'false')
        options.setdefault('fcgi', 'false')
        options.setdefault('wsgilog', '')
        options.setdefault('logfile', '')

        # only try to download stuff if we aren't asked to install from cache
        self.install_from_cache = self.buildout['buildout'].get(
            'install-from-cache', '').strip() == 'true'


    def install(self):
        base_dir = self.buildout['buildout']['directory']

        project_dir = os.path.join(base_dir, self.options['project'])

        extra_paths = self.get_extra_paths()
        requirements, ws = self.egg.working_set(['cykooz.djangorecipe'])

        script_paths = []

        # Create the Django management script
        script_paths.extend(self.create_manage_script(extra_paths, ws))

        # Create the test runner
        script_paths.extend(self.create_test_runner(extra_paths, ws))

        # Make the wsgi and fastcgi scripts if enabled
        script_paths.extend(self.make_scripts(extra_paths, ws))

        # Create default settings if we haven't got a project
        # egg specified, and if it doesn't already exist
        if not self.options.get('projectegg'):
            if not os.path.exists(project_dir):
                self.create_project(project_dir)
            else:
                self.log.info(
                    'Skipping creating of project: %(project)s since '
                    'it exists' % self.options)

        return script_paths


    def create_manage_script(self, extra_paths, ws):
        project = self.options.get('projectegg', self.options['project'])
        return zc.buildout.easy_install.scripts(
            [(self.options.get('control-script', self.name),
              'cykooz.djangorecipe.manage', 'main')],
            ws, self.options['executable'], self.options['bin-directory'],
            extra_paths = extra_paths,
            arguments= "'%s.%s'" % (project,
                                    self.options['settings']))


    def create_test_runner(self, extra_paths, working_set):
        apps = self.options.get('test', '').split()
        # Only create the testrunner if the user requests it
        if apps:
            return zc.buildout.easy_install.scripts(
                [(self.options.get('testrunner', 'test'),
                  'cykooz.djangorecipe.test', 'main')],
                working_set, self.options['executable'],
                self.options['bin-directory'],
                extra_paths = extra_paths,
                arguments= "'%s.%s', %s" % (
                    self.options['project'],
                    self.options['settings'],
                    ', '.join(["'%s'" % app for app in apps])))
        else:
            return []


    def create_project(self, project_dir):
        old_cwd = os.getcwd()
        try:
            os.chdir(self.buildout['buildout']['directory'])
            startproject.Command().run_from_argv(['django-admin.py', 'startproject', self.options['project']])
        finally:
            os.chdir(old_cwd)
        
        os.remove(os.path.join(project_dir, 'manage.py'))

        template_vars = {}
        template_vars.update(self.options)

        self.create_file(
            os.path.join(project_dir, 'development.py'),
            development_settings, template_vars)

        self.create_file(
            os.path.join(project_dir, 'production.py'),
            production_settings, template_vars)

        settings_path = os.path.join(project_dir, 'settings.py')
        settings = open(settings_path, 'r').read()
        
        # Add the necessary imports
        settings = 'import os\n' + settings
        
        # Set default value for TEMPLATE_DIRS setting
        def_template_dirs = '''os.path.join(os.path.dirname(__file__), 'templates')'''
        settings = re.sub(r'(TEMPLATE_DIRS\s?=\s?\()', '\\1\n    %s,' % def_template_dirs,
                          settings, re.MULTILINE)

        # Change ADMIN_MEDIA_PREFIX setting
        settings = re.sub(r"(ADMIN_MEDIA_PREFIX\s*?=\s*?')/media/(')", r'\1/admin_media/\2',
                          settings, re.MULTILINE)

        # Change MEDIA_ROOT setting
        settings = re.sub(r"(MEDIA_ROOT\s*?=\s*?)''", r'\1%s' % self.options['media_root'],
                          settings, re.MULTILINE)

        # Change MEDIA_URL setting
        settings = re.sub(r"(MEDIA_URL\s*?=\s*?')(')", r'\1/media/\2',
                          settings, re.MULTILINE)
        
        open(settings_path, 'w').write(settings)
        
        # Create the media and templates directories for our project
        os.mkdir(os.path.join(project_dir, 'media'))
        os.mkdir(os.path.join(project_dir, 'templates'))


    def make_scripts(self, extra_paths, ws):
        scripts = []
        _script_template = zc.buildout.easy_install.script_template
        for protocol in ('wsgi', 'fcgi'):
            zc.buildout.easy_install.script_template = \
                zc.buildout.easy_install.script_header + \
                    script_template[protocol]
            if self.options.get(protocol, '').lower() == 'true':
                project = self.options.get('projectegg',
                                           self.options['project'])
                if protocol == 'wsgi' and 'wsgi-script' in self.options:
                    filename = os.path.basename(self.options['wsgi-script'])
                    dirname = os.path.dirname(self.options['wsgi-script'])
                    dirname = dirname or self.options['bin-directory']
                else:    
                    filename = '%s.%s' % (self.options.get('control-script', self.name), protocol)
                    dirname = self.options['bin-directory']
                scripts.extend(
                    zc.buildout.easy_install.scripts(
                        [(filename, 'cykooz.djangorecipe.%s' % protocol, 'main')],
                        ws,
                        self.options['executable'],
                        dirname,
                        extra_paths=extra_paths,
                        arguments= "'%s.%s', logfile='%s'" % (
                            project, self.options['settings'],
                            self.options.get('logfile'))))
        zc.buildout.easy_install.script_template = _script_template
        return scripts


    def get_extra_paths(self):
        extra_paths = [self.buildout['buildout']['directory']
                       ]

        # Add libraries found by a site .pth files to our extra-paths.
        if 'pth-files' in self.options:
            import site
            for pth_file in self.options['pth-files'].splitlines():
                pth_libs = site.addsitedir(pth_file, set())
                if not pth_libs:
                    self.log.warning(
                        "No site *.pth libraries found for pth_file=%s" % (
                         pth_file,))
                else:
                    self.log.info("Adding *.pth libraries=%s" % pth_libs)
                    self.options['extra-paths'] += '\n' + '\n'.join(pth_libs)

        pythonpath = [p.replace('/', os.path.sep) for p in
                      self.options['extra-paths'].splitlines() if p.strip()]

        extra_paths.extend(pythonpath)
        return extra_paths


    def update(self):
        extra_paths = self.get_extra_paths()
        requirements, ws = self.egg.working_set(['cykooz.djangorecipe'])
        # Create the Django management script
        self.create_manage_script(extra_paths, ws)

        # Create the test runner
        self.create_test_runner(extra_paths, ws)

        # Make the wsgi and fastcgi scripts if enabled
        self.make_scripts(extra_paths, ws)


    def create_file(self, file, template, options):
        if os.path.exists(file):
            return

        f = open(file, 'w')
        f.write(template % options)
        f.close()

